import unittest
import numpy as np
from flask_monitoringdashboard.core.reporting.mean_permutation_test import mean_permutation_test


class TestMeanPermutationTest(unittest.TestCase):
    def setUp(self):
        np.random.seed(0)  # We want tests to be deterministic

    def test_different_random_distributions(self):
        x = np.random.randint(0, 16, 100)  # 100 random numbers between 0 and 16
        y = np.random.randint(0, 10, 100)  # 100 random numbers between 0 and 10

        p = mean_permutation_test(x, y)

        self.assertLess(
            p, 0.05
        )  # p < 0.05 -> reject null hypothesis that distributions come from same distribution

    def test_different_normal_distributions(self):
        x = np.random.normal(5, 10, 100)  # mu=5, sd=10
        y = np.random.normal(0, 10, 100)  # mu=0, sd=10

        p = mean_permutation_test(x, y)

        self.assertLess(
            p, 0.05
        )  # p < 0.05 -> reject null hypothesis that distributions come from same distribution

    def test_equal_normal_distributions(self):
        x = np.random.normal(0, 10, 100)  # mu=0, sd=10
        y = np.random.normal(0, 10, 100)  # ditto

        p = mean_permutation_test(x, y)

        self.assertGreater(p, 0.05)

    def test_equal_random_distributions(self):
        x = np.random.randint(0, 10, 100)  # 100 random numbers between 0 and 10
        y = np.random.randint(0, 10, 100)  # ditto

        p = mean_permutation_test(x, y)

        self.assertGreater(p, 0.05)




from abc import ABCMeta, abstractmethod


class Answer:
    __metaclass__ = ABCMeta

    def __init__(self, type):
        self.type = type

    @abstractmethod
    def is_significant(self):
        pass

    @abstractmethod
    def meta(self):
        pass

    def serialize(self):
        base = dict(
            is_significant=self.is_significant(),
            type=self.type
        )
        base.update(self.meta())

        return base


class ReportQuestion:
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_answer(self, endpoint, comparison_interval, compared_to_interval):
        pass



import unittest

from flask_monitoringdashboard.database import session_scope
from flask_monitoringdashboard.database.count_group import (
    count_requests_group,
    count_requests_per_day,
)
from flask_monitoringdashboard.test.utils import (
    set_test_environment,
    clear_db,
    add_fake_data,
    TIMES,
)


class TestCountGroup(unittest.TestCase):
    def setUp(self):
        set_test_environment()
        clear_db()
        add_fake_data()

    def test_count_requests_group(self):
        with session_scope() as db_session:
            self.assertEqual(count_requests_group(db_session), [(1, 5)])

    def test_count_requests_per_day(self):
        with session_scope() as db_session:
            self.assertEqual(count_requests_per_day(db_session, []), [])

            if TIMES[0].date() == TIMES[len(TIMES) - 1].date():  # must all be on the same day
                self.assertEqual(count_requests_per_day(db_session, [TIMES[0].date()]), [[(1, 5)]])



import unittest
from time import sleep

from flask import Flask, json, jsonify
from flask_monitoringdashboard.core.cache import EndpointInfo
from flask_monitoringdashboard.database import session_scope, Request

from flask_monitoringdashboard.test.utils import (
    set_test_environment,
    clear_db
)


def get_test_app_for_status_code_testing(schedule=False):
    """
    :return: Flask Test Application with the right settings
    """
    import flask_monitoringdashboard

    app = Flask(__name__)

    @app.route('/return-a-simple-string')
    def return_a_simple_string():
        return 'Hello, world'

    @app.route('/return-a-tuple')
    def return_a_tuple():
        return 'Hello, world', 404

    @app.route('/ridiculous-return-value')
    def return_ridiculous_return_value():
        return 'hello', 'ridiculous'

    @app.route('/return-jsonify-default-status-code')
    def return_jsonify_default_status_code():
        return jsonify({
            'apples': 'banana'
        })

    @app.route('/return-jsonify-with-custom-status-code')
    def return_jsonify_with_custom_status_code():
        response = jsonify({
            'cheese': 'pears'
        })
        response.status_code = 401
        return response

    @app.route('/unhandled-exception')
    def unhandled_exception():
        potatoes = 1000
        bananas = 0

        return potatoes / bananas

    app.config['SECRET_KEY'] = flask_monitoringdashboard.config.security_token
    app.testing = True
    flask_monitoringdashboard.user_app = app
    app.config['WTF_CSRF_ENABLED'] = False
    app.config['WTF_CSRF_METHODS'] = []
    flask_monitoringdashboard.config.get_group_by = lambda: '12345'
    flask_monitoringdashboard.bind(app=app, schedule=schedule)
    TEST_CACHE = {'main': EndpointInfo()}
    flask_monitoringdashboard.core.cache.memory_cache = TEST_CACHE
    return app


class TestLogin(unittest.TestCase):
    def setUp(self):
        set_test_environment()
        clear_db()
        self.app = get_test_app_for_status_code_testing()

    def test_simple_string_response(self):
        """
            An endpoint that just returns a string yields a HTTP 200 status code and should be logged as such.
        """
        with self.app.test_client() as c:
            c.get('/return-a-simple-string')

            with session_scope() as db_session:
                requests = db_session.query(Request.status_code).all()

                self.assertEqual(len(requests), 1)
                self.assertEqual(requests[0][0], 200)

    def test_return_a_tuple(self):
        """
        An endpoint that returns a tuple should log the second parameter as status_code
        """
        with self.app.test_client() as c:
            c.get('/return-a-tuple')

            with session_scope() as db_session:
                requests = db_session.query(Request.status_code, Request.endpoint_id).all()

                self.assertEqual(len(requests), 1)
                self.assertEqual(requests[0][0], 404)

    def test_jsonify_default_status_code(self):
        """
        An endpoint that returns a Response as a return value of jsonify without setting the status_cod yields a HTTP
        200 status and should be logged as such.
        """
        with self.app.test_client() as c:
            c.get('/return-jsonify-default-status-code')

            with session_scope() as db_session:
                requests = db_session.query(Request.status_code, Request.endpoint_id).all()

                self.assertEqual(len(requests), 1)
                self.assertEqual(requests[0][0], 200)

    def test_jsonify_with_custom_status_code(self):
        """
        An endpoint that returns a Response and has a custom status code assigned should properly log the specified
        status code
        """
        with self.app.test_client() as c:
            c.get('/return-jsonify-with-custom-status-code')

            with session_scope() as db_session:
                requests = db_session.query(Request.status_code, Request.endpoint_id).all()

                self.assertEqual(len(requests), 1)
                self.assertEqual(requests[0][0], 401)

    def test_ridiculous_return_value(self):
        """
        An endpoint that returns a silly status code like a string should yield a 500 status code
        """
        with self.app.test_client() as c:
            c.get('/ridiculous-return-value')

            with session_scope() as db_session:
                requests = db_session.query(Request.status_code, Request.endpoint_id).all()

                self.assertEqual(len(requests), 1)
                self.assertEqual(requests[0][0], 500)

    def test_unhandled_exception(self):
        """
        An endpoint that returns a silly status code like a string should yield a 500 status code
        """
        with self.app.test_client() as c:
            try:
                c.get('/unhandled-exception')
            except:
                pass

            sleep(.5)

            with session_scope() as db_session:
                requests = db_session.query(Request.status_code, Request.endpoint_id).all()

                self.assertEqual(len(requests), 1)
                self.assertEqual(requests[0][0], 500)



from flask_monitoringdashboard.core.reporting.mean_permutation_test import mean_permutation_test
import numpy as np

from flask_monitoringdashboard.core.reporting.questions.report_question import Answer, ReportQuestion
from flask_monitoringdashboard.database import session_scope

from flask_monitoringdashboard.database.request import get_latencies_sample


class AverageLatencyAnswer(Answer):
    def __init__(self, is_significant, comparison_interval_latencies_sample=None,
                 compared_to_interval_latencies_sample=None, percentual_diff=None, comparison_interval_avg=None,
                 compared_to_interval_avg=None):
        super().__init__('AVERAGE_LATENCY')

        self._is_significant = is_significant
        self._comparison_interval_latencies_sample = comparison_interval_latencies_sample
        self._compared_to_interval_latencies_sample = compared_to_interval_latencies_sample
        self._percentual_diff = percentual_diff

        self._compared_to_interval_avg = compared_to_interval_avg
        self._comparison_interval_avg = comparison_interval_avg

    def meta(self):
        return dict(
            latencies_sample=dict(
                comparison_interval=self._comparison_interval_latencies_sample,
                compared_to_interval=self._compared_to_interval_latencies_sample
            ),
            comparison_average=self._comparison_interval_avg,
            compared_to_average=self._compared_to_interval_avg,
            percentual_diff=self._percentual_diff,
        )

    def is_significant(self):
        return self._is_significant


class AverageLatency(ReportQuestion):

    def get_answer(self, endpoint, comparison_interval, compared_to_interval):
        with session_scope() as db_session:
            comparison_interval_latencies_sample = get_latencies_sample(db_session, endpoint.id, comparison_interval)
            compared_to_interval_latencies_sample = get_latencies_sample(db_session, endpoint.id, compared_to_interval)

            if len(comparison_interval_latencies_sample) == 0 or len(compared_to_interval_latencies_sample) == 0:
                return AverageLatencyAnswer(is_significant=False)

            comparison_interval_avg = np.average(comparison_interval_latencies_sample)
            compared_to_interval_avg = np.average(compared_to_interval_latencies_sample)

            percentual_diff = (comparison_interval_avg - compared_to_interval_avg) / compared_to_interval_avg * 100

            p_value = mean_permutation_test(comparison_interval_latencies_sample,
                                            compared_to_interval_latencies_sample,
                                            num_rounds=1000)
            is_significant = abs(float(percentual_diff)) > 30 and p_value < 0.05

            return AverageLatencyAnswer(
                is_significant=is_significant,

                percentual_diff=percentual_diff,

                # Sample latencies
                comparison_interval_latencies_sample=comparison_interval_latencies_sample,
                compared_to_interval_latencies_sample=compared_to_interval_latencies_sample,

                # Latency averages
                comparison_interval_avg=comparison_interval_avg,
                compared_to_interval_avg=compared_to_interval_avg
            )
